searchData={"items":[{"type":"module","title":"Pachka","doc":"Pachka is a message batching library for Elixir applications.\n\nIt provides a reliable way to collect messages and deliver them in batches to configurable destinations.\nMessages are buffered until either the batch size limit is reached or the batch timeout occurs.\nFailed deliveries are automatically retried with configurable backoff strategies.","ref":"Pachka.html"},{"type":"module","title":"Features - Pachka","doc":"* Configurable batch sizes and timeouts\n  * Customizable message sinks for different delivery targets\n  * Automatic retries with customizable backoff\n  * Overload protection with queue size limits\n  * Graceful shutdown with message draining","ref":"Pachka.html#module-features"},{"type":"module","title":"Example - Pachka","doc":"defmodule MyApp.MessageSink do\n      @behaviour Pachka.Sink\n      \n      @impl true\n      def send_batch(messages, _server_value) do\n        # Process messages in batch\n        :ok\n      end\n    end\n\n    # Start Pachka server\n    {:ok, pid} = Pachka.start_link(\n      name: MyPachka,\n      sink: MyApp.MessageSink,\n      max_batch_size: 100,\n      max_batch_delay: :timer.seconds(1)\n    )\n\n    # Send messages\n    :ok = Pachka.send_message(MyPachka, %{event: \"user_login\"})\n    :ok = Pachka.send_message(MyPachka, %{event: \"page_view\"})\n\nThe messages will be collected and delivered to the sink in batches based on the configured\nbatch size and delay parameters.","ref":"Pachka.html#module-example"},{"type":"module","title":"Shutdown - Pachka","doc":"When a Pachka server receives a shutdown request (via `Pachka.stop/2` or from its parent `Supervisor`), it enters termination mode. During termination, Pachka stops accepting new messages and processes all accumulated messages.\n\nIf `Pachka.Sink` does not implement `c:Pachka.Sink.drain_on_terminate/3`, Pachka processes accumulated messages using its standard operation flow - dividing the queue into batches and invoking `c:Pachka.Sink.send_batch/2` for each batch. On batch failure, Pachka initiates retries with intervals determined by `c:Pachka.Sink.retry_timeout/3`.\n\nIf `Pachka.Sink` implements `c:Pachka.Sink.drain_on_terminate/3`, the Pachka server transfers control to this function along with all accumulated messages.\n\n> #### `c:Pachka.Sink.drain_on_terminate/3` {: .tip}\n> You can skip implementing `c:Pachka.Sink.drain_on_terminate/3` when export failure risk is minimal - for instance, when storing messages in an SQLite database on the local machine's disk.\n> \n> For all other scenarios, implement `c:Pachka.Sink.drain_on_terminate/3` with a fallback system to prevent message loss during network outages or target system failures.","ref":"Pachka.html#module-shutdown"},{"type":"module","title":"In-flight export - Pachka","doc":"When a Pachka server enters termination mode during an ongoing export, it allows this export to complete normally within the time defined by `export_timeout`. This is a deliberate decision based on the notion that the intersection of two normal operations - batch export and process termination should not result in an abnormal operation - export cancellation before `export_timeout` expires.\n\nIf the in-flight export fails, unsent messages are returned to the queue and the entire queue is passed to `c:Pachka.Sink.drain_on_terminate/3`.","ref":"Pachka.html#module-in-flight-export"},{"type":"module","title":"Shutdown timeout - Pachka","doc":"Pachka does not define a time limit for termination mode. The actual termination time limit is set by the requesting system - either through the `timeout` parameter of `Pachka.stop/2` or via the `:shutdown` option when launched through a `Supervisor` (see [Shutdown values (:shutdown)](https://hexdocs.pm/elixir/Supervisor.html#module-shutdown-values-shutdown)).\n\n> #### Set the termination timeout based on the worst-case scenario {: .tip}\n> - messages to send = `critical_queue_size` messages in queue + `max_batch_size` unsent messages from in-flight export\n> - time to send them = `shutdown timeout` - `export_timeout` duration spent by the failed export","ref":"Pachka.html#module-shutdown-timeout"},{"type":"module","title":"Startup and shutdown order - Pachka","doc":"When using Pachka in your supervision tree, the order of children is important for proper startup and shutdown behavior:\n\n1. Start the sink system first\n2. Start Pachka servers\n3. Start message producers last\n\nThis ordering ensures:\n\n- During startup: The sink system is ready to receive messages before Pachka begins processing\n- During shutdown: Message producers stop first, then Pachka drains remaining messages, and finally the sink system terminates\n\nExample configuration:\n\n    children = [\n      # 1. Sink system (e.g. database connection)\n      MyApp.Repo,\n      \n      # 2. Pachka server\n      {Pachka, name: MyPachka, sink: MyApp.MessageSink},\n      \n      # 3. Message producers\n      MyApp.UserTracker,\n      MyApp.MetricsCollector\n    ]\n\n    Supervisor.start_link(children, strategy: :one_for_one)\n\nThis order prevents message loss during system startup and shutdown since supervisors start children sequentially and shut them down in reverse order (see [Start and shutdown](https://hexdocs.pm/elixir/Supervisor.html#module-start-and-shutdown)).","ref":"Pachka.html#module-startup-and-shutdown-order"},{"type":"function","title":"Pachka.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Pachka.html#child_spec/1"},{"type":"type","title":"Pachka.message/0","doc":"A message that can be sent to a `Pachka` server process.","ref":"Pachka.html#t:message/0"},{"type":"type","title":"Pachka.option/0","doc":"Option values used by the `start_link/1` function.","ref":"Pachka.html#t:option/0"},{"type":"type","title":"Pachka.options/0","doc":"Options used by the `start_link/1` function.","ref":"Pachka.html#t:options/0"},{"type":"function","title":"Pachka.send_message/2","doc":"Sends a message to a `Pachka` server process.\n\nThe message will be added to the queue and eventually delivered to the configured sink. \nIf the server's queue is full, the message will be rejected.","ref":"Pachka.html#send_message/2"},{"type":"function","title":"Parameters - Pachka.send_message/2","doc":"* `name` - The name or pid of the Pachka server process\n  * `message` - The message to be sent","ref":"Pachka.html#send_message/2-parameters"},{"type":"function","title":"Return values - Pachka.send_message/2","doc":"Returns `:ok` if the message was successfully queued, or `{:error, :overloaded}` if the server's queue is full.","ref":"Pachka.html#send_message/2-return-values"},{"type":"function","title":"Examples - Pachka.send_message/2","doc":"iex> Pachka.send_message(MyPachka, %{event: \"user_login\"})\n    :ok\n\n    iex> Pachka.send_message(pid, \"message\") \n    :ok","ref":"Pachka.html#send_message/2-examples"},{"type":"function","title":"Pachka.start_link/1","doc":"Starts a `Pachka` server process linked to the current process.","ref":"Pachka.html#start_link/1"},{"type":"function","title":"Options - Pachka.start_link/1","doc":"* `:sink` (`t:module/0`) - Required. The module that implements the `Pachka.Sink` behavior for processing batched messages.\n\n* `:name` (`t:GenServer.name/0`) - The name to register the Pachka server process under. See [Name Registration](https://hexdocs.pm/elixir/GenServer.html#module-name-registration) for details.\n\n* `:server_value` (`t:Pachka.Sink.server_value/0`) - Value passed by the server process to `Pachka.Sink` callback functions. Can be used to distinguish between different Pachka processes using the same sink module or to store server-specific configuration. The default value is `nil`.\n\n* `:start_link_opts` (`t:GenServer.options/0`) - Options passed to `GenServer.start_link/3`\n\n* `:max_batch_size` (`t:pos_integer/0`) - Maximum number of messages to accumulate before forcing a batch export. Batches sent to `c:Pachka.Sink.send_batch/2` are guaranteed to be no larger than that size. The default value is `500`.\n\n* `:critical_queue_size` (`t:pos_integer/0`) - Maximum number of messages the server can hold before it starts rejecting new ones. The default value is `10000`.\n\n* `:max_batch_delay` (`t:timeout/0`) - Maximum time to wait before exporting a batch. The default value is `5000`.\n\n* `:export_timeout` (`t:timeout/0`) - Maximum time allowed for the `c:Pachka.Sink.send_batch/2` to process a batch. The default value is `10000`.","ref":"Pachka.html#start_link/1-options"},{"type":"function","title":"Return values - Pachka.start_link/1","doc":"If the server is successfully created and initialized, this function returns `{:ok, pid}`, where `pid` is the PID of the server.\nIf a process with the specified server name already exists, this function returns `{:error, {:already_started, pid}}` with the PID of that process.\n\nRaises a `NimbleOptions.ValidationError` exception if `opts` are not valid.","ref":"Pachka.html#start_link/1-return-values"},{"type":"function","title":"Pachka.stop/2","doc":"Synchronously stops the server.\n\nRefer to the [Shutdown](#module-shutdown) section for information about server termination.","ref":"Pachka.html#stop/2"},{"type":"behaviour","title":"Pachka.Sink","doc":"Behaviour module for implementing message sinks in Pachka.\n\nA sink is responsible for processing batches of messages collected by Pachka.\nIt defines how messages should be delivered to their destination and handles retry logic when delivery fails.","ref":"Pachka.Sink.html"},{"type":"behaviour","title":"Example - Pachka.Sink","doc":"defmodule MySink do\n      @behaviour Pachka.Sink\n\n      @impl true\n      def send_batch(messages, _server_value) do\n        # Process the batch of messages\n        :ok\n      end\n\n      @impl true\n      def retry_timeout(retry_num, _failure_reason, _server_value) do\n        # Exponential backoff\n        :timer.seconds(retry_num * 2)\n      end\n    end","ref":"Pachka.Sink.html#module-example"},{"type":"callback","title":"Pachka.Sink.drain_on_terminate/3","doc":"Handles remaining messages during Pachka server termination.\n\nCalled when the Pachka server is stopping and there are still messages\nin the queue. The implementation can choose how to handle these messages\nto prevent data loss.","ref":"Pachka.Sink.html#c:drain_on_terminate/3"},{"type":"callback","title":"Parameters - Pachka.Sink.drain_on_terminate/3","doc":"* `messages` - List of remaining messages\n  * `batch_size` - Maximum batch size configured for the server\n  * `server_value` - Custom value configured when starting Pachka\n\n> #### Optional callback {: .info}\n>\n> If one is not implemented, Pachka will process remaining messages normally","ref":"Pachka.Sink.html#c:drain_on_terminate/3-parameters"},{"type":"type","title":"Pachka.Sink.failure_reason/0","doc":"Reason for message delivery failure.\n\nCan be either a custom error from `send_batch/2` or `:killed` when\nthe export process is terminated due to timeout.","ref":"Pachka.Sink.html#t:failure_reason/0"},{"type":"callback","title":"Pachka.Sink.retry_timeout/3","doc":"Determines the timeout before retrying a failed batch.\n\nCalled when `send_batch/2` returns an error or when the export process is killed\ndue to timeout. The implementation can use the retry attempt number and failure\nreason to implement custom backoff strategies.","ref":"Pachka.Sink.html#c:retry_timeout/3"},{"type":"callback","title":"Parameters - Pachka.Sink.retry_timeout/3","doc":"* `retry_num` - Number of retry attempts made so far (starts from 1)\n  * `failure_reason` - Reason why the previous attempt failed\n  * `server_value` - Custom value configured when starting Pachka","ref":"Pachka.Sink.html#c:retry_timeout/3-parameters"},{"type":"callback","title":"Returns - Pachka.Sink.retry_timeout/3","doc":"Timeout in milliseconds before the next retry attempt.\n\n> #### Optional callback {: .info}\n>\n> Default implementation will wait for a number of seconds equal to retry number","ref":"Pachka.Sink.html#c:retry_timeout/3-returns"},{"type":"callback","title":"Pachka.Sink.send_batch/2","doc":"Processes a batch of messages.\n\nThis callback is called when Pachka has collected enough messages to form a batch\nor when the batch timeout is reached. The implementation should deliver these messages to their destination.","ref":"Pachka.Sink.html#c:send_batch/2"},{"type":"callback","title":"Parameters - Pachka.Sink.send_batch/2","doc":"* `messages` - List of messages to be processed\n  * `server_value` - Custom value configured when starting Pachka","ref":"Pachka.Sink.html#c:send_batch/2-parameters"},{"type":"callback","title":"Returns - Pachka.Sink.send_batch/2","doc":"* `:ok` - if messages were successfully processed\n  * `{:error, reason}` - if processing failed","ref":"Pachka.Sink.html#c:send_batch/2-returns"},{"type":"type","title":"Pachka.Sink.send_error/0","doc":"Error returned by `send_batch/2` when message delivery fails.","ref":"Pachka.Sink.html#t:send_error/0"},{"type":"type","title":"Pachka.Sink.server_value/0","doc":"Custom value that is passed to all sink callbacks.\n\nCan be used to distinguish between different Pachka processes using the same sink module or to store server-specific configuration.\nConfigured in `Pachka.start_link/1`.","ref":"Pachka.Sink.html#t:server_value/0"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}